@@ -22,35 +22,56 @@ import argparse
 import json
 import os
 import sys
+import re
 
-from .models import (load_model, quantize, dump_config, save_weights, load_weights,
+from .layers import QuantizationParams
+from .models import (load_model, quantize, dump_config,
                      check_quantization, insert_rescaling)
 
 
-def quantize_model(model_path, quant_config, add_deq=True, output_name=None):
+def quantize_model(model_path, quant_config, qparams, add_deq=True, output_name=None):
     """ CLI entry point to quantize a model using the provided configuration.
 
     Args:
         model_path (str): Path to the model to quantize.
         quant_config (str): Path to the quantization configuration file.
+        qparams (QuantizationParams): global quantization parameters
         add_deq (bool, optional): allows to convert output to float. Defaults to True.
         output_name (str, optional): name of the output quantized model. If none provided
             set a default name as <model>_<config>.h5. Defaults to None.
     """
     # Build name for the output model
     model_name = os.path.splitext(model_path)[0]
-    config_name = os.path.splitext(os.path.basename(quant_config))[0]
-    if output_name is None:
-        output_name = f"{model_name}_{config_name}.h5"
 
-    # Load the configuration file and the model
-    with open(quant_config) as f:
-        config = json.load(f)
+    # Set default output_name using the config when there is one and the qparams otherwise
+    if output_name is None:
+        if quant_config:
+            output_name = f"{model_name}_{os.path.splitext(os.path.basename(quant_config))[0]}.h5"
+        else:
+            # Quantization suffix has a well-identified structure
+            suffix_re = r"(_i\d_w\d_a\d)"
+            p = re.compile(suffix_re)
+            # Look for an existing quantization suffix in the base name
+            match = p.match(model_name)
+            if match:
+                # Only keep the actual base name (group(2) contains the suffix)
+                model_name = match.group(1)
+            output_name = f"{model_name}_i{qparams.input_weight_bits}_w{qparams.weight_bits}"\
+                          f"_a{qparams.activation_bits}.h5"
+
+    # Load the configuration file
+    if quant_config:
+        with open(quant_config) as f:
+            config = json.load(f)
+    else:
+        config = None
+
+    # Load the model
     model = load_model(model_path)
 
     # Quantize the model and save it
     print(f"Quantizing model {model_path} with configuration file {quant_config}.")
-    model_q = quantize(model, config, add_deq)
+    model_q = quantize(model, q_config=config, qparams=qparams, add_dequantizer=add_deq)
     model_q.save(output_name, include_optimizer=False)
     print(f"Saved quantized model to {output_name}.")
 
@@ -76,39 +97,6 @@ def dump_model_config(model_path, output_name=None):
     print(f"Saved quantization configuration to {output_name}.")
 
 
-def save_model_weights(model_path, weights_path):
-    """ CLI entry point to save the model weights in an npz file.
-
-    Args:
-        model_path (str): Path to the model to extract the weights from.
-        weights_path (str): Path to save the npz file.
-            Defaults to <model_path>.npz.
-    """
-    # Build name for weights file
-    if weights_path is None:
-        model_name = os.path.splitext(model_path)[0]
-        weights_path = f"{model_name}.npz"
-
-    # Load the model and save its weights
-    model = load_model(model_path)
-    save_weights(model, weights_path)
-    print(f"Saved model weights to {weights_path}.")
-
-
-def load_model_weights(model_path, weights_path):
-    """ CLI entry point to apply weights to a model from an npz file.
-
-    Args:
-        model_path (str): Path to the model on which apply the weights.
-        weights_path (str): Path to load the npz file.
-    """
-    # Update the model weights with the npz file
-    model = load_model(model_path)
-    load_weights(model, weights_path)
-    model.save(model_path, include_optimizer=False)
-    print(f"Saved model with new weights to {model_path}.")
-
-
 def check_quantized_model(model_path):
     """ CLI entry point to check model quantization.
 
@@ -150,10 +138,18 @@ def main():
     q_parser = sp.add_parser(
         "quantize", help="Quantize an input model, given a quantization configuration file.")
     q_parser.add_argument("-m", "--model", type=str, required=True, help="Model to quantize")
-    q_parser.add_argument("-c", "--quantization_config", type=str,
-                          required=True, help="Quantization configuration file")
+    q_parser.add_argument("-c", "--quantization_config", type=str, default=None,
+                          help="Quantization configuration file")
     q_parser.add_argument("-nd", "--no_deq", action="store_false",
                           help="Do not add a dequantizer after head")
+    q_parser.add_argument("-a", "--activation_bits", type=int, default=4,
+                          help="Activation quantization bitwidth")
+    q_parser.add_argument("--per_tensor_activations", action='store_true',
+                          help="Quantize activations per-tensor")
+    q_parser.add_argument("-w", "--weight_bits", type=int, default=4,
+                          help="Weight quantization bitwidth")
+    q_parser.add_argument("-i", "--input_weight_bits", type=int, default=8,
+                          help="Input layer weight quantization bitwidth")
     q_parser.add_argument("-o", "--out_name", type=str,
                           help="Output quantized model name.")
 
@@ -164,20 +160,6 @@ def main():
     c_parser.add_argument("-o", "--output_path", type=str, help="Store quantization configuration. "
                           "Defaults to <model>_quant_config.json")
 
-    # Save weights arguments
-    w_parser = sp.add_parser("save_weights", help="Store models weights in an npz.")
-    w_parser.add_argument("-m", "--model", type=str, required=True,
-                          help="Model to extract weights from.")
-    w_parser.add_argument("-w", "--weights_path", type=str,
-                          help="Npz file that contains the saved weights.")
-
-    # Load weights arguments
-    w_parser = sp.add_parser("load_weights", help="Apply weights to a model from an npz file.")
-    w_parser.add_argument("-m", "--model", type=str, required=True,
-                          help="Model to apply the weights.")
-    w_parser.add_argument("-w", "--weights_path", type=str, required=True,
-                          help="Npz file that contains the weights to apply.")
-
     # Check action and arguments
     c_parser = sp.add_parser(
         "check", help="Check the quantization of a model to detect issues that can be fixed "
@@ -207,9 +189,14 @@ def main():
             import importlib_metadata as metadata
         print(metadata.version('quantizeml'))
     elif args.action == "quantize":
+        qparams = QuantizationParams(activation_bits=args.activation_bits,
+                                     per_tensor_activations=args.per_tensor_activations,
+                                     weight_bits=args.weight_bits,
+                                     input_weight_bits=args.input_weight_bits)
         quantize_model(
             model_path=args.model,
             quant_config=args.quantization_config,
+            qparams=qparams,
             add_deq=args.no_deq,
             output_name=args.out_name
         )
@@ -218,16 +205,6 @@ def main():
             model_path=args.model,
             output_name=args.output_path,
         )
-    elif args.action == "save_weights":
-        save_model_weights(
-            model_path=args.model,
-            weights_path=args.weights_path,
-        )
-    elif args.action == "load_weights":
-        load_model_weights(
-            model_path=args.model,
-            weights_path=args.weights_path,
-        )
     elif args.action == "check":
         check_quantized_model(args.model)
     elif args.action == "insert_rescaling":
