@@ -14,18 +14,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ******************************************************************************
+
+__all__ = ["PaddedConv2D", "QuantizedConv2D", "QuantizedConv2DTranspose"]
+
 import tensorflow as tf
 
 from keras import layers, backend
 from keras.utils import conv_utils
 
-from .recorders import TensorRecorder
-from .quantizers import (WeightQuantizer, AlignedWeightQuantizer,
-                         OutputQuantizer)
-from ..tensors import QTensor, QFloat, FixedPoint, MAX_BUFFER_BITWIDTH
-
-
-__all__ = ["PaddedConv2D", "QuantizedConv2D", "QuantizedConv2DTranspose"]
+from .decorators import (register_quantize_target, rescale_outputs, tensor_inputs,
+                         neural_layer_init, register_aligned_inputs)
+from .quantizers import AlignedWeightQuantizer
+from ..tensors import QTensor, QFloat, FixedPoint
 
 
 def apply_padding(inputs, strides, kernel_size, padding_value):
@@ -106,6 +106,8 @@ class PaddedConv2D(layers.Conv2D):
         return config
 
 
+@register_quantize_target([layers.Conv2D, PaddedConv2D])
+@register_aligned_inputs
 @tf.keras.utils.register_keras_serializable()
 class QuantizedConv2D(layers.Conv2D):
     """A convolutional layer that operates on quantized inputs and weights.
@@ -114,28 +116,12 @@ class QuantizedConv2D(layers.Conv2D):
     value (overriding 'padding' parameter).
 
     Args:
-        quant_config (dict): the serialized quantization configuration.
+        quant_config (dict, optional): the serialized quantization configuration. Defaults to None.
         padding_value (float, optional): the value used when padding for the 'same' convolution
             type. If None, zero-padding is used. Defaults to None.
     """
-
-    def __init__(self, *args, padding_value=None, quant_config, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.quant_config = quant_config
-        out_quant_cfg = quant_config.get("output_quantizer", False)
-        if out_quant_cfg:
-            self.out_quantizer = OutputQuantizer(
-                name="output_quantizer", **out_quant_cfg)
-        else:
-            self.out_quantizer = None
-        self.weight_quantizer = WeightQuantizer(
-            name="weight_quantizer", **quant_config["weight_quantizer"])
-        if self.use_bias:
-            self.bias_quantizer = AlignedWeightQuantizer(
-                name="bias_quantizer", **quant_config["bias_quantizer"])
-        self.buffer_bitwidth = self.quant_config.get(
-            "buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
-        assert self.buffer_bitwidth > 0, "The buffer_bitwidth must be a strictly positive integer."
+    @neural_layer_init(False)
+    def __init__(self, *args, padding_value=None, quant_config=None, **kwargs):
         if padding_value is None:
             self.padding_value = None
         else:
@@ -150,31 +136,9 @@ class QuantizedConv2D(layers.Conv2D):
             # will be done explicitely in the call.
             self.padding = 'valid'
 
-        # Add objects that will store the shift values.
-        self.input_shift = TensorRecorder()
-
+    @tensor_inputs([QTensor, tf.Tensor])
+    @rescale_outputs
     def call(self, inputs):
-        # raise an error if the inputs are not QTensor or tf.Tensor
-        if not isinstance(inputs, (QTensor, tf.Tensor)):
-            raise TypeError(f"QuantizedConv2D only accepts QTensor or tf.Tensor\
-                              inputs. Receives {type(inputs)} inputs.")
-
-        # Quantize the weights
-        kernel = self.weight_quantizer(self.kernel)
-
-        if isinstance(inputs, tf.Tensor):
-            # Assume the inputs are integer stored as float, which is the only tf.Tensor
-            # inputs that are allowed
-            inputs = FixedPoint(inputs, 8, 0).promote(self.buffer_bitwidth)
-
-        if isinstance(inputs, FixedPoint):
-            # Expand the inputs to a higher bitwidth to avoid saturation and align them
-            inputs, shift = inputs.expand(self.buffer_bitwidth)
-            self.input_shift(shift)
-        else:
-            # Just promote the QFloat inputs to avoid a saturation
-            inputs = inputs.promote(self.buffer_bitwidth)
-
         # Quantize the weights
         kernel = self.weight_quantizer(self.kernel)
 
@@ -212,8 +176,6 @@ class QuantizedConv2D(layers.Conv2D):
             bias = self.bias_quantizer(self.bias, outputs)
             outputs = tf.add(outputs, bias)
 
-        if self.out_quantizer is not None:
-            outputs = self.out_quantizer(outputs)
         return outputs
 
     def get_config(self):
@@ -224,40 +186,22 @@ class QuantizedConv2D(layers.Conv2D):
         return config
 
 
+@register_quantize_target(layers.Conv2DTranspose)
+@register_aligned_inputs
 @tf.keras.utils.register_keras_serializable()
 class QuantizedConv2DTranspose(layers.Conv2DTranspose):
     """A transposed convolutional layer that operates on quantized inputs and weights.
 
     Args:
-        quant_config (dict): the serialized quantization configuration.
+        quant_config (dict, optional): the serialized quantization configuration. Defaults to None.
     """
+    @neural_layer_init(False)
+    def __init__(self, *args, quant_config=None, **kwargs):
+        pass
 
-    def __init__(self, *args, quant_config, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.quant_config = quant_config
-        out_quant_cfg = quant_config.get("output_quantizer", False)
-        if out_quant_cfg:
-            self.out_quantizer = OutputQuantizer(
-                name="output_quantizer", **out_quant_cfg)
-        else:
-            self.out_quantizer = None
-        self.weight_quantizer = WeightQuantizer(
-            name="weight_quantizer", **quant_config["weight_quantizer"])
-        if self.use_bias:
-            self.bias_quantizer = AlignedWeightQuantizer(
-                name="bias_quantizer", **quant_config["bias_quantizer"])
-        self.buffer_bitwidth = self.quant_config.get("buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
-        assert self.buffer_bitwidth > 0, "The buffer_bitwidth must be a strictly positive integer."
-
-        # Add object that will store the shift values.
-        self.input_shift = TensorRecorder()
-
+    @tensor_inputs([FixedPoint])
+    @rescale_outputs
     def call(self, inputs):
-        # Raise an error if the inputs are not FixedPoint
-        if not isinstance(inputs, FixedPoint):
-            raise TypeError("QuantizedConv2DTranspose only accepts FixedPoint inputs. "
-                            f"Receives {type(inputs)} inputs.")
-
         # Quantize the weights
         # Conv2DTranspose weight shape is (h, w, filters, channels) which cannot be quantized
         # 'per-axis' as it is: the last dimension is not the output last dimension (i.e filters)
@@ -266,10 +210,6 @@ class QuantizedConv2DTranspose(layers.Conv2DTranspose):
         # Tensorflow operation.
         kernel = self.weight_quantizer(tf.transpose(self.kernel, (0, 1, 3, 2)))
 
-        # Expand the inputs to a higher bitwidth to avoid saturation and align them
-        inputs, shift = inputs.expand(self.buffer_bitwidth)
-        self.input_shift(shift)
-
         # Prepare deconvolution output shape
         inputs_shape = tf.shape(inputs)
         batch_size, height, width = inputs_shape[0], inputs_shape[1], inputs_shape[2]
@@ -309,8 +249,6 @@ class QuantizedConv2DTranspose(layers.Conv2DTranspose):
             # Add biases
             outputs = tf.add(outputs, bias)
 
-        if self.out_quantizer is not None:
-            outputs = self.out_quantizer(outputs)
         return outputs
 
     def get_config(self):
