@@ -14,27 +14,26 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ******************************************************************************
+
+__all__ = ["QuantizedDropout"]
+
 import keras
 import tensorflow as tf
 
+from .decorators import register_quantize_target, tensor_inputs, register_no_output_quantizer
 from ..tensors import QTensor
 
 
-__all__ = ["QDropout"]
-
-
+@register_quantize_target(keras.layers.Dropout)
+@register_no_output_quantizer
 @tf.keras.utils.register_keras_serializable()
-class QDropout(keras.layers.Dropout):
+class QuantizedDropout(keras.layers.Dropout):
     """ A dropout layer that operates on quantized inputs and weights.
 
     It is only implemented as a passthrough.
     """
 
+    @tensor_inputs([QTensor])
     def call(self, inputs):
-        # raise an error if the inputs are not QTensor
-        if not isinstance(inputs, QTensor):
-            raise TypeError(f"QDropout only accepts QTensor inputs.\
-                             Receives {type(inputs)} inputs.")
-
         # QuantizedDropout act as a pass through.
         return inputs
