@@ -14,6 +14,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ******************************************************************************
+
+__all__ = ["OutputQuantizer"]
+
 import tensorflow as tf
 import keras.backend as K
 from contextlib import contextmanager
@@ -24,9 +27,6 @@ from ..recorders import TensorRecorder, FixedPointRecorder
 from .quantizers import Quantizer
 
 
-__all__ = ["OutputQuantizer"]
-
-
 @contextmanager
 def disable_partitioner(layer):
     partitioner = None
@@ -49,28 +49,26 @@ class OutputQuantizer(Calibrable, Quantizer):
     inspired by the BatchNormalizationLayer moving average algorithm.
 
     Args:
-        bitwidth (int): the quantization bitwidth.
+        bitwidth (int, optional): the quantization bitwidth. Defaults to 8.
         signed (bool, optional): whether the quantizer expects signed values or unsigned.
             Defaults to True.
         axis (str, optional): the quantization range is a scalar ('per-tensor') or a vector
             corresponding to the last axis ('per-axis'). Defaults to 'per-tensor'.
         momentum (float, optional): the momentum for the moving average. Defaults to 0.9.
         scale_bits: (int, optional): the bitwidth to use when quantizing output scales.
-            Defaults to None.
+            Defaults to 8.
     """
 
-    def __init__(self, bitwidth, signed=True, axis="per-tensor",
-                 momentum=0.9, scale_bits=None, **kwargs):
+    def __init__(self, bitwidth=8, signed=True, axis="per-tensor",
+                 momentum=0.9, scale_bits=8, **kwargs):
         super().__init__(bitwidth, signed, **kwargs)
         self._axis = axis
         if not (isinstance(axis, str) and axis in ["per-tensor", "per-axis"]):
             raise ValueError(f"Only support reduction 'per-tensor' or 'per-axis'. Given {axis}.")
         self.momentum = momentum
+        self.scale_bits = scale_bits
         # Add object that will store the shift values.
         self.shift = TensorRecorder()
-        self.scale_bits = scale_bits
-        if scale_bits is not None:
-            self.qscales = FixedPointRecorder()
 
     def build(self, input_shape):
         """Build the layer.
@@ -176,6 +174,11 @@ class OutputQuantizer(Calibrable, Quantizer):
                 raise ValueError(f"{self.name} receives QFloat inputs: the scale_bits parameter"
                                  " needs to be specified.")
             inputs, qscales = inputs.to_fixed_point(self.scale_bits)
+            if getattr(self, 'qscales', None) is None:
+                # From Keras documentation, any variable creation taking place in call
+                # should be wrapped with tf.init_scope
+                with tf.init_scope():
+                    self.qscales = FixedPointRecorder()
             self.qscales(qscales)
 
         if inputs.value_bits <= self.value_bits:
