@@ -14,15 +14,21 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ******************************************************************************
-import tensorflow as tf
-import keras
-
-from ...tensors import FixedPoint
-from ...debugging import assert_equal
 
 __all__ = ["QuantizedPermute"]
 
+import tensorflow as tf
+import keras
 
+from ..decorators import (register_quantize_target, tensor_inputs, register_no_output_quantizer,
+                          register_aligned_inputs)
+from ...tensors import FixedPoint
+from ...debugging import assert_equal
+
+
+@register_quantize_target(keras.layers.Permute)
+@register_no_output_quantizer
+@register_aligned_inputs
 @tf.keras.utils.register_keras_serializable()
 class QuantizedPermute(keras.layers.Permute):
     """A Permute layer that operates on quantized inputs
@@ -36,11 +42,8 @@ class QuantizedPermute(keras.layers.Permute):
             of the input.
     """
 
+    @tensor_inputs([FixedPoint])
     def call(self, inputs):
-        # raise an error if the inputs are not FixedPoint
-        if not isinstance(inputs, FixedPoint):
-            raise TypeError(f"QuantizedPermute only accepts FixedPoint inputs.\
-                             Receives {type(inputs)} inputs.")
         if not inputs.per_tensor:
             # Different fractional-bits are defined for the last axis, so
             # it must be preserved during the transposition
