@@ -14,15 +14,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ******************************************************************************
-import tensorflow as tf
-import keras
-
-from ...tensors import QTensor, FixedPoint
-
 
 __all__ = ["QuantizedReshape"]
 
+import tensorflow as tf
+import keras
 
+from ..decorators import register_quantize_target, tensor_inputs, register_no_output_quantizer
+from ...tensors import QTensor, FixedPoint
+
+
+@register_quantize_target(keras.layers.Reshape)
+@register_no_output_quantizer
 @tf.keras.utils.register_keras_serializable()
 class QuantizedReshape(keras.layers.Reshape):
     """A Reshape layer that operates on quantized inputs
@@ -32,11 +35,8 @@ class QuantizedReshape(keras.layers.Reshape):
             dimension (batch size).
     """
 
+    @tensor_inputs([QTensor])
     def call(self, inputs):
-        # raise an error if the inputs are not QTensor
-        if not isinstance(inputs, QTensor):
-            raise TypeError(f"QuantizedReshape only accepts QTensor inputs.\
-                             Receives {type(inputs)} inputs.")
         # Return a new reshaped QTensor
         result = tf.reshape(inputs, (tf.shape(inputs)[0],) + self.target_shape)
         if not tf.executing_eagerly():
