@@ -17,8 +17,9 @@
 
 import tensorflow as tf
 
-from quantizeml.layers import WeightQuantizer, OutputQuantizer, QuantizedReLU
-from quantizeml.tensors import QFloat, ceil_log2, floor_log2
+from ..layers import WeightQuantizer, OutputQuantizer, QuantizedReLU, QFloatRecorder
+from ..tensors import ceil_log2, floor_log2
+from .record import record_quantization_variables
 
 
 def check_quantization(model):
@@ -35,37 +36,14 @@ def check_quantization(model):
     Returns:
         list(str): the quantization issues that were detected
     """
+    # First record the model variables to store qscales in output quantizers
+    record_quantization_variables(model)
+
     messages = []
     for layer in model.layers:
-        for name, attr in layer.__dict__.items():
-            if isinstance(attr, WeightQuantizer):
-                quantizer = attr
-                if name == "bias_quantizer":
-                    w = layer.bias
-                elif name == "dw_weight_quantizer":
-                    w = layer.depthwise_kernel
-                elif name == "pw_weight_quantizer":
-                    w = layer.pointwise_kernel
-                elif name == "beta_quantizer":
-                    w = layer.get_weights()[1]
-                else:
-                    w = layer.get_weights()[0]
-                w_q = quantizer(w)
-                if isinstance(w_q, QFloat):
-                    axis = list(range(len(w.shape) - 1))
-                    max_values = tf.reduce_max(tf.abs(w), axis)
-                    ideal_scales = QFloat.optimal_scales(max_values, quantizer.value_bits)
-                    err = tf.abs(ideal_scales - w_q.scales) / tf.abs(ideal_scales)
-                    mean_err = tf.reduce_mean(err)
-                    if mean_err > 5e-2:
-                        message = f"Scales quantization relative error is high in " \
-                                    f"{layer.name}/{quantizer.name}: {mean_err:.4f}."
-                        if quantizer._axis == "per-tensor":
-                            message += "Use a per-axis quantizer and/or increase scales bits."
-                        else:
-                            message += "Try increasing scales bits."
-                        messages.append(message)
-            elif isinstance(attr, OutputQuantizer):
+        for _, attr in layer.__dict__.items():
+            if isinstance(attr, OutputQuantizer):
+                # Check staturation in integer operations
                 quantizer = attr
                 max_value = quantizer.get_weights()[0]
                 if tf.reduce_all(max_value == 1):
@@ -85,4 +63,31 @@ def check_quantization(model):
                 if tf.reduce_any(pot_max_value):
                     messages.append(f"Saturation detected before {layer.name}/{quantizer.name}: "
                                     "check the previous operation and/or increase buffer bitwidth.")
+
+                # Check inaccurate weight scales quantization.
+                qscales = getattr(quantizer, 'qscales', None)
+                if qscales:
+                    # Retrieve the original scales. They are located in the WeightQuantizer of the
+                    # layer if there is one, or in all WeightQuantizer if they are several (eg.
+                    # SeparableConvolution)
+                    ideal_scales = None
+                    for _, attr in layer.__dict__.items():
+                        if (isinstance(attr, WeightQuantizer) and
+                                isinstance(attr.qweights, QFloatRecorder)):
+                            if ideal_scales is None:
+                                ideal_scales = attr.qweights.value.scales
+                            else:
+                                ideal_scales *= attr.qweights.value.scales
+
+                    if ideal_scales is not None:
+                        err = tf.abs(ideal_scales - qscales.value.to_float()) / tf.abs(ideal_scales)
+                        mean_err = tf.reduce_mean(err)
+                        if mean_err > 5e-2:
+                            message = f"Scales quantization relative error is high in " \
+                                      f"{layer.name}/{quantizer.name}: {mean_err:.4f}."
+                            if quantizer._axis == "per-tensor":
+                                message += "Use a per-axis quantizer and/or increase scales bits."
+                            else:
+                                message += "Try increasing scales bits."
+                            messages.append(message)
     return messages
