@@ -21,21 +21,21 @@ from keras.layers import (ReLU, Conv2D, Conv2DTranspose, SeparableConv2D, Depthw
                           Dropout, MaxPool2D, GlobalAvgPool2D, Reshape, Flatten)
 
 from .. import deep_clone_model, get_layers_by_type, find_layer_config
-from ...layers import PaddedConv2D, SeparableConv2DTranspose
+from ...layers import PaddedConv2D, DepthwiseConv2DTranspose, QuantizedReLU
 from ...tensors import FixedPoint, floor_log2
 
 
 def _get_surrounding_layer(layer, previous, supported_layers, skippable_layers):
-    """ Finds the layer preceeding or following a target layer.
+    """ Finds the layer preceding or following a target layer.
 
     Args:
         layer (keras.Layer): the layer of interest
-        previous (bool): True to find the preceeding layer, False to find the layer that follows
+        previous (bool): True to find the preceding layer, False to find the layer that follows
         supported_layers (list): layer type that will support ReLU alignement
         skippable_layers (list): layer types that can be skipped
 
     Returns:
-        keras.Layer: the layer preceeding or the layer following the layer of interest if valid,
+        keras.Layer: the layer preceding or the layer following the layer of interest if valid,
         None otherwise
     """
     if previous:
@@ -52,17 +52,17 @@ def _get_surrounding_layer(layer, previous, supported_layers, skippable_layers):
         surrounding_layer = outbounds[0].layer
 
     # If the layer is supported, it is a valid candidate
-    if type(surrounding_layer) in supported_layers:
+    if isinstance(surrounding_layer, supported_layers):
         return surrounding_layer
     # If the surrounding layer can be skipped, recursively call the function
-    elif type(surrounding_layer) in skippable_layers:
+    elif isinstance(surrounding_layer, skippable_layers):
         return _get_surrounding_layer(surrounding_layer, previous, supported_layers,
                                       skippable_layers)
     # If the surrounding layer is not supported, alignement cannot happen
     return None
 
 
-def align_relu_max_value(model, q_config):
+def align_relu_max_value(model, q_config=None):
     """ Update the ReLU activations in the model so that their max values are closer to the maximum
     integer value the OutputQuantizer can use given in the quantization configuration.
 
@@ -80,24 +80,27 @@ def align_relu_max_value(model, q_config):
 
     Args:
         model (keras.Model): the model to update
-        q_config (dict): quantization configuration
+        q_config (dict, optional): quantization configuration, should be provided if the model is
+            not quantized. Defaults to None.
 
     Returns:
         keras.Model: the model with ReLU activations updated.
     """
     # Define layers that will support weight rescaling induced by ReLU alignement
     supported_layers = (Conv2D, PaddedConv2D, Conv2DTranspose, SeparableConv2D, DepthwiseConv2D,
-                        Dense, SeparableConv2DTranspose)
+                        Dense, DepthwiseConv2DTranspose)
 
     # Define layers that can be skippped when looking for layers to compensate alignement
-    skippable_layers = (Dropout, MaxPool2D, GlobalAvgPool2D, Reshape, Flatten)
+    skippable_layers = Dropout, MaxPool2D, GlobalAvgPool2D, Reshape, Flatten
 
     # Clone the model to apply modifications
     clone = deep_clone_model(model)
 
     # Get all ReLU layers present in the model and proceed to alignement
-    relus = get_layers_by_type(clone, ReLU)
+    relus = get_layers_by_type(clone, (ReLU, QuantizedReLU))
     for relu in relus:
+        if relu.max_value is None:
+            continue
         # Retrieve the surrounding layers: inbound and outbound layers must be compatible, meaning
         # they have weights where a rescaling will happen
         previous_layer = _get_surrounding_layer(relu, True, supported_layers, skippable_layers)
@@ -106,7 +109,11 @@ def align_relu_max_value(model, q_config):
             continue
 
         # Get the ReLU quantization configuration
-        relu_config = find_layer_config(relu.name, q_config)
+        if isinstance(relu, ReLU):
+            relu_config = find_layer_config(relu.name, q_config)
+        else:
+            relu_config = relu.get_config()['quant_config']
+
         if (relu_config is None or relu_config.get('output_quantizer', None) is None or
                 relu_config.get('output_quantizer').get('bitwidth', None) is None):
             continue
