@@ -28,8 +28,8 @@ from keras.layers import (BatchNormalization, MaxPool2D, GlobalAvgPool2D, Conv2D
 from ..utils import (deep_clone_model, _get_layers, _get_layer_index, _inbound_node_generator,
                      _replace_layer_name_for_connection_info)
 from ...layers import (PaddedConv2D, QuantizedConv2D, QuantizedSeparableConv2D, QuantizedDense,
-                       QuantizedDepthwiseConv2D, QuantizedConv2DTranspose, SeparableConv2DTranspose,
-                       QuantizedSeparableConv2DTranspose)
+                       QuantizedDepthwiseConv2D, QuantizedConv2DTranspose, DepthwiseConv2DTranspose,
+                       QuantizedDepthwiseConv2DTranspose)
 
 
 def invert_batchnorm_pooling(model):
@@ -146,8 +146,8 @@ def _find_batchnorms_to_fold(model):
 
     supported_layers = (Conv2D, PaddedConv2D, Conv2DTranspose, SeparableConv2D, DepthwiseConv2D,
                         Dense, QuantizedConv2D, QuantizedConv2DTranspose, QuantizedSeparableConv2D,
-                        QuantizedDepthwiseConv2D, QuantizedDense, SeparableConv2DTranspose,
-                        QuantizedSeparableConv2DTranspose)
+                        QuantizedDepthwiseConv2D, QuantizedDense, DepthwiseConv2DTranspose,
+                        QuantizedDepthwiseConv2DTranspose)
 
     # Map between a layer and its following BN layer and map between the BN layer and its next layer
     map_prev_layer_to_bn = {}
@@ -259,7 +259,7 @@ def _compute_BN_folded_weights(target_layer, bn_layer):
     # Get kernel and bias weights of the target layer
     kernel_position = 0
     bias_position = 1
-    if isinstance(target_layer, SeparableConv2DTranspose):
+    if isinstance(target_layer, DepthwiseConv2DTranspose):
         # Pointwise kernel and bias are last in the list, the first weights are the C depthwise
         # kernels
         kernel_position = -2 if target_layer.use_bias else -1
