@@ -18,22 +18,20 @@
 Common utility methods used in quantization models.
 """
 
+__all__ = ['load_model', 'deep_clone_model', 'insert_layer',
+           'get_layers_by_type', 'find_layer_config', 'insert_rescaling']
+
 import re
 import warnings
 from copy import deepcopy
-from pathlib import Path
 
-import numpy as np
 from keras.models import clone_model, load_model as kload_model, Model, Sequential
 from keras.layers import serialize, Layer, InputLayer, Rescaling
 
 from ..layers.quantizers import OutputQuantizer, Dequantizer
 
-__all__ = ['load_model', 'deep_clone_model', 'insert_layer', 'load_weights', 'save_weights',
-           'get_layers_by_type', 'find_layer_config', 'insert_rescaling']
 
-
-def load_model(model_path, custom_layers={}, compile_model=True):
+def load_model(model_path, custom_layers=None, compile_model=True):
     """Loads a model with Vision Transformer custom layers.
 
     Args:
@@ -230,44 +228,6 @@ def apply_weights_to_model(model, weights, verbose=True):
             warnings.warn(f"Variable '{key}' not found in the model.")
 
 
-def load_weights(model, weights_path):
-    """Loads weights from a npz file and apply it to a model.
-
-    Go through the dictionary of weights of the npz file, find the corresponding variable in the
-    model and partially load its weights.
-
-    Args:
-        model (keras.Model): the model to update
-        weights_path (str): the path of the npz file to load
-    """
-    # Check the npz file validity
-    path = Path(weights_path)
-    if not path.is_file():
-        raise ValueError(f"File `{weights_path}` not found.")
-
-    # Open the npz file
-    weights_dict = np.load(weights_path)
-
-    # Apply the weights to the model
-    apply_weights_to_model(model, weights_dict)
-
-
-def save_weights(model, weights_path):
-    """Save model weights on an npz file.
-
-    Takes a model and save the weights of all its layers into an npz file.
-
-    Args:
-        model (keras.Model): the model to save its weights
-        weights_path (str): the path of the npz file to save
-    """
-    weights_dict = {}
-    for var in model.variables:
-        weights_dict[var.name] = var
-
-    np.savez(weights_path, **weights_dict)
-
-
 def _get_layers(config, layer_names):
     """Extracts layers from a model configuration.
 
@@ -363,16 +323,17 @@ def get_layers_by_type(model, layer_type):
     return layers
 
 
-def find_layer_config(layer_name, qconfig):
+def find_layer_config(layer_name, qconfig=None):
     """ Extract the layer quantization parameters from the overall quantization config file.
 
     Args:
         layer_name (str): the target layer name
-        qconfig (dict): quantization configuration
+        qconfig (dict, optional): quantization configuration. Defaults to None.
 
     Returns:
         dict: layer quantization configuration
     """
+    qconfig = qconfig or dict()
     # Extract configuration matching patterns
     patterns = list(qconfig.keys())
 
