@@ -123,9 +123,7 @@ class QFloat(QTensor):
         """
         # Evaluate QFloat scales fractional bits, without clamping to scale_bits
         scales_frac_bits = FixedPoint.max_frac_bits(scale_bits, scales, clamp=False)
-        # Quantize the scales "up" to make sure they are at least equal to the actual scales.
-        # Should they be lower, we would not cover the full quantization range.
-        return FixedPoint.quantize(scales, scale_bits, scales_frac_bits, ceil=True)
+        return FixedPoint.quantize(scales, scale_bits, scales_frac_bits)
 
     @staticmethod
     def quantize(x, value_bits, scales, frac_bits=0.):
