@@ -53,20 +53,18 @@ def ceil_through(x):
 
 class QTensor(tf.experimental.ExtensionType):
     """Abstract class to exchange quantized tensors between layers
-
-    The QTensor values are actually stored as integer, but it provides a
-    conversion method to project these values into a float representation.
-
-    The value_bits parameter sets the maximum integer values that can be stored:
-
-        int_max = 2^bits - 1.
-
-    When a QTensor is created, its values are clipped to [-int_max-1, int_max].
     """
-    values: tf.Tensor = 1.0
-    value_bits: int = 7
     shape: tf.TensorShape  # Required to convert to a KerasTensor
 
+    @property
+    def name(self):
+        """Returns the QTensor name
+
+        Returns:
+            str: the QTensor name
+        """
+        raise NotImplementedError
+
     @property
     def per_tensor(self):
         """Returns if QTensor is quantized per-tensor
@@ -93,8 +91,9 @@ class QTensor(tf.experimental.ExtensionType):
         return copy(self)
 
     def __str__(self):
+        class_name = self.__class__.__name__
         x_float = self.to_float()
-        return f"QTensor: {x_float}"
+        return f"{class_name}: {x_float}"
 
     @staticmethod
     def int_max(value_bits):
@@ -107,8 +106,7 @@ class QTensor(tf.experimental.ExtensionType):
 
     def assert_per_tensor(self):
         """Asserts that a QTensor is quantized per-tensor"""
-        name = self.__class__.__name__ if not hasattr(self.values, "name") else self.values.name
-        assert_equal(self.per_tensor, True, message=f"{name} is not per-tensor.")
+        assert_equal(self.per_tensor, True, message=f"{self.name} is not per-tensor.")
 
 
 def pow2(n):
