@@ -119,6 +119,10 @@ def fp_qf_matmul(a: FixedPoint,
     Returns:
         :obj:`QFloat`: the multiplied QFloat.
     """
+    # If b is transposed, its last dimension will collapse, so it must be per-tensor
+    if transpose_b:
+        b.assert_per_tensor()
+
     # Do matmul on a and b inner FixedPoint
     fp_prod = tf.matmul(a, b.fp, transpose_a, transpose_b, adjoint_a, adjoint_b, name)
 
@@ -126,6 +130,54 @@ def fp_qf_matmul(a: FixedPoint,
     return QFloat(fp_prod, b.scales)
 
 
+@tf.experimental.dispatch_for_api(tf.linalg.matmul)
+def qf_qf_matmul(a: QFloat,
+                 b: QFloat,
+                 transpose_a=False,
+                 transpose_b=False,
+                 adjoint_a=False,
+                 adjoint_b=False,
+                 a_is_sparse=False,
+                 b_is_sparse=False,
+                 output_type=None,
+                 name=None):
+    """Multiplies matrix `a` by matrix `b`, producing `a` * `b`.
+
+    Args:
+        a (:obj:`QFloat`): a QFloat of rank > 1.
+        b (:obj:`QFloat`): a QFloat with same rank as `a`.
+        transpose_a (bool, optional): if `True`, `a` is transposed before multiplication.
+            Defaults to False.
+        transpose_b (bool, optional): if `True`, `b` is transposed before multiplication.
+            Defaults to False.
+        adjoint_a (bool, optional): if `True`, `a` is conjugated and transposed before
+            multiplication. Defaults to False.
+        adjoint_b (bool, optional): if `True`, `b` is conjugated and transposed before
+            multiplication. Defaults to False.
+        a_is_sparse (bool, optional): must be False, argument kept for compatibility with
+            original tf.matmul. Defaults to False.
+        b_is_sparse (bool, optional): must be False, argument kept for compatibility with
+            original tf.matmul. Defaults to False.
+        output_type (NoneType, optional): must be None, argument kept for compatibility
+            with original tf.matmul. Defaults to None.
+        name (str, optional): the name for the operation. Defaults to None.
+
+    Returns:
+        :obj:`QFloat`: the multiplied QFloat.
+    """
+    # the first input must be per-tensor to be able to multiply the scales
+    a.assert_per_tensor()
+    # The second input must also be per-tensor if it is transposed
+    if transpose_b:
+        b.assert_per_tensor()
+
+    # Do matmul on a and b inner FixedPoint
+    fp_prod = tf.matmul(a.fp, b.fp, transpose_a, transpose_b, adjoint_a, adjoint_b, name)
+
+    # Return a new QFloat with multiplied scales
+    return QFloat(fp_prod, a.scales * b.scales)
+
+
 @tf.experimental.dispatch_for_api(keras.backend.depthwise_conv2d)
 def fp_fp_depthwise_conv2d(x: FixedPoint,
                            depthwise_kernel: FixedPoint,
