@@ -130,10 +130,11 @@ def fp_concat(values: List[FixedPoint], axis, name="concat"):
     if len(values) == 1:
         return FixedPoint(values[0].values, values[0].value_bits, values[0].frac_bits)
 
-    # Only support inputs with the same value bits
-    assert_equal(values[0].value_bits, values[1].value_bits, "We only support concatenation of "
-                 f"inputs with same value_bits. Receives {values[0].value_bits} and "
-                 f"{values[1].value_bits}.")
+    # Promote to the higher value bits
+    if values[0].value_bits < values[1].value_bits:
+        values[0] = values[0].promote(values[1].value_bits)
+    elif values[0].value_bits > values[1].value_bits:
+        values[1] = values[1].promote(values[0].value_bits)
 
     # For now we don't support concatenation over last dimension
     rank_input = values[0].values.shape.ndims
